# -*- coding: utf-8 -*-
"""
Created in Jul 2022

@author: Ernest Namdar
"""
import sys
import numpy as np
from pypwr.cohen_ES import cohen_ES
from scipy.stats import t
from scipy.stats import nct
from scipy.stats import norm
from scipy.optimize import brentq

def pwr_t_test(alternative, test_type, n=None, d=None, sig_level=0.05, power=None):
    assert alternative in ["two.sided","less","greater"]
    assert test_type in ["two.sample", "one.sample", "paired"]
    alternative_options = ["less", "two.sided", "greater"]
    test_type_options = ["one.sample", "two.sample", "paired"]
    if sum(x is None for x in [n, d, power, sig_level]) != 1:
        raise Exception("exactly one of n, d, power, and sig.level must be NULL")
        sys.exit(0)
    if (d is not None) and isinstance(d,str):
        d = cohen_ES(test="t",size=d)["effect_size"]
    if (sig_level is not None):
        #print(type(sig_level))
        if type(sig_level)!=int and type(sig_level)!=float:
            raise Exception("sig_level should be numerical")
            sys.exit(0)
        elif sig_level<0 or sig_level>1:
            raise Exception("sig_level should be in [0, 1]")
            sys.exit(0)
    if (power is not None):
        if type(power)!=int and type(power)!=float:
            raise Exception("power should be numerical")
            sys.exit(0)
        elif power<0 or power>1:
            raise Exception("power should be in [0, 1]")
            sys.exit(0)
    tsample = test_type_options.index(test_type)
    if tsample == 2:
        tsample=0
    tsample +=1
    ttside = alternative_options.index(alternative)
    if ttside == 1:
        tside = 1
    else:
        tside=0
    tside += 1 #to avoid div by 0
    if tside==2 and (d is not None):
        d = np.abs(d)
    if ttside == 0:
        p_body = "nct.cdf(t.ppf(sig_level/tside, df=(n-1)*tsample), df=(n-1)*tsample, nc=np.sqrt(n/tsample)*d)"
        # nu = (n-1)*tsample
        # qu = t.ppf(sig_level/tside, df=nu)
        # nct.cdf(qu, df=nu, nc=np.sqrt(n/tsample)*d)
    elif ttside == 1:
        p_body = "nct.sf(t.isf(sig_level/tside, df=(n-1)*tsample), df=(n-1)*tsample, nc=np.sqrt(n/tsample)*d)+nct.cdf(-(t.isf(sig_level/tside, df=(n-1)*tsample)), df=(n-1)*tsample, nc=np.sqrt(n/tsample)*d)"
        # nu = (n-1)*tsample
        # qu = t.isf(sig_level/tside, df=nu)
        # nct.sf(qu, df=nu, nc=np.sqrt(n/tsample)*d)+nct.cdf(-qu, df=nu, nc=np.sqrt(n/tsample)*d)
    elif ttside == 2:
        p_body = "nct.sf(t.isf(sig_level/tside, df=(n-1)*tsample), df=(n-1)*tsample, nc=np.sqrt(n/tsample)*d)"
        # nu = (n-1)*tsample
        # qu = t.isf(sig_level/tside, df=nu)
        # nct.sf(qu, df=nu, nc=np.sqrt(n/tsample)*d)
    if power is None:
        power = eval(p_body)
    elif d is None:
        loc = {}
        target_function_def = "def target_function(d):return "+p_body+"-power"
        exec(target_function_def, loc)
        #print(loc)
        loc.update(globals())
        loc.update(locals())
        if tside-1==0:
            d = brentq(loc["target_function"], -10, 5)
        elif tside-1==1:
            d = brentq(loc["target_function"], 1e-07, 10)
        elif tside-1==2:
            d = brentq(loc["target_function"], -5, 10)
    elif n is None:
        loc = {}
        target_function_def = "def target_function(n):return "+p_body+"-power"
        exec(target_function_def, loc)
        #print(loc)
        loc.update(globals())
        loc.update(locals())
        n = brentq(loc["target_function"], 2+1e-10, 1e+09)
    elif sig_level is None:
        loc = {}
        target_function_def = "def target_function(sig_level):return "+p_body+"-power"
        exec(target_function_def, loc)
        #print(loc)
        loc.update(globals())
        loc.update(locals())
        sig_level = brentq(loc["target_function"], 1e-10, 1-1e-10)
    else:
        raise Exception("internal error")
        sys.exit(0)
    if test_type=="paired":
        NOTE = "n is number of *pairs*"
        METHOD = "Paired t test power calculation"
    elif test_type=="two.sample":
        NOTE = "n is number in *each* group"
        METHOD = "Two-sample t test power calculation"
    else:
        NOTE = ""
        METHOD = "One-sample t test power calculation"
    # print("h=",h, "n=", n, "power=",power, "sig_level=", sig_level)
    return {"n":n, "d":d, "sig_level":sig_level, "power":power,
            "alternative":alternative, "method":METHOD, "note":NOTE}



# -*- coding: utf-8 -*-
"""
Created in Jul 2022

@author: Ernest Namdar
"""
import numpy as np

def ES_w2(P):
    pi = np.sum(P, axis=0)
    pj = np.sum(P, axis=1)
    P0 = np.matmul(pi,pj)
    return np.sqrt(np.sum((P-P0)**2/P0))



# -*- coding: utf-8 -*-
"""
Created in Jul 2022

@author: Ernest Namdar
"""
import sys
import numpy as np
from pypwr.cohen_ES import cohen_ES
from scipy.stats import f as f_dist
from scipy.stats import ncf
from scipy.optimize import brentq

def pwr_f2_test(u=None, v=None, f2=None, sig_level=0.05, power=None):
    if sum(x is None for x in [u, v, f2, power, sig_level]) != 1:
        raise Exception("exactly one of u, v, f2, power, and sig.level must be NULL")
        sys.exit(0)
    if f2 is not None:
        if isinstance(f2,str):
            f2 = cohen_ES(test="f2",size=f2)["effect_size"]
        if f2<0:
            raise Exception("f2 must be positive")
            sys.exit(0)
    if (u is not None) and u<1:
        raise Exception("degree of freedom u for numerator must be at least 1")
        sys.exit(0)
    if (v is not None) and v<1:
        raise Exception("degree of freedom v for denominator must be at least 1")
        sys.exit(0)
    if (sig_level is not None):
        #print(type(sig_level))
        if type(sig_level)!=int and type(sig_level)!=float:
            raise Exception("sig_level should be numerical")
            sys.exit(0)
        elif sig_level<0 or sig_level>1:
            raise Exception("sig_level should be in [0, 1]")
            sys.exit(0)
    if (power is not None):
        if type(power)!=int and type(power)!=float:
            raise Exception("power should be numerical")
            sys.exit(0)
        elif power<0 or power>1:
            raise Exception("power should be in [0, 1]")
            sys.exit(0)
    p_body = "ncf.sf(f_dist.isf(sig_level, dfn=u, dfd=v), dfn=u, dfd=v, nc=f2*(u+v+1))"
    if power is None:
        power = eval(p_body)
    elif u is None:
        loc = {}
        target_function_def = "def target_function(u):return "+p_body+"-power"
        exec(target_function_def, loc)
        #print(loc)
        loc.update(globals())
        loc.update(locals())
        u = brentq(loc["target_function"], 1+1e-10, 100)
    elif v is None:
        loc = {}
        target_function_def = "def target_function(v):return "+p_body+"-power"
        exec(target_function_def, loc)
        #print(loc)
        loc.update(globals())
        loc.update(locals())
        v = brentq(loc["target_function"], 1+1e-10, 1e+09)
    elif f2 is None:
        loc = {}
        target_function_def = "def target_function(f2):return "+p_body+"-power"
        exec(target_function_def, loc)
        #print(loc)
        loc.update(globals())
        loc.update(locals())
        f2 = brentq(loc["target_function"], 1e-07, 1e+07)
    elif sig_level is None:
        loc = {}
        target_function_def = "def target_function(sig_level):return "+p_body+"-power"
        exec(target_function_def, loc)
        #print(loc)
        loc.update(globals())
        loc.update(locals())
        sig_level = brentq(loc["target_function"], 1e-10, 1-1e-10)
    else:
        raise Exception("internal error")
        sys.exit(0)
    METHOD = "Multiple regression power calculation"
    # print("h=",h, "n=", n, "power=",power, "sig_level=", sig_level)
    return {"u":u, "v":v, "f2":f2, "sig_level":sig_level, "power":power,
            "method":METHOD}



# -*- coding: utf-8 -*-
"""
Created in Jul 2022

@author: Ernest Namdar
"""
def cohen_ES(test, size):
    """

    Parameters
    ----------
    test : str
        Available options are "p","t","r","anov","chisq","f2".
    size : str
        Available options are "small","medium","large".

    Returns
    -------
    dict
        a dictionary of test, size, effect_size (calculated), and method.

    """
    assert test in ["p","t","r","anov","chisq","f2"]
    assert size in ["small","medium","large"]
    test_options = ["p","t","r","anov","chisq","f2"]
    # size_options = ["small","medium","large"]
    ntest = test_options.index(test)
    if ntest == 0:
        size_mapping = {"small":0.2, "medium":0.5, "large":0.8}
        ES = size_mapping[size]
    elif ntest == 1:
        size_mapping = {"small":0.2, "medium":0.5, "large":0.8}
        ES = size_mapping[size]
    elif ntest == 2:
        size_mapping = {"small":0.1, "medium":0.3, "large":0.5}
        ES = size_mapping[size]
    elif ntest == 3:
        size_mapping = {"small":0.1, "medium":0.25, "large":0.4}
        ES = size_mapping[size]
    elif ntest == 4:
        size_mapping = {"small":0.1, "medium":0.3, "large":0.5}
        ES = size_mapping[size]
    elif ntest == 5:
        size_mapping = {"small":0.02, "medium":0.15, "large":0.35}
        ES = size_mapping[size]
    METHOD = "Conventional effect size from Cohen (1982)"
    return {"test":test, "size":size, "effect_size":ES, "method":METHOD}




#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Python version of the pwr R package
@author: Ernest Namdar  ernest.namdar@utoronto.ca
"""
from pypwr.ES_h import ES_h
from pypwr.ES_w1 import ES_w1
from pypwr.ES_w2 import ES_w2
from pypwr.cohen_ES import cohen_ES
from pypwr.pwr_t_test import pwr_t_test
from pypwr.pwr_t2n_test import pwr_t2n_test
from pypwr.pwr_2p_test import pwr_2p_test
from pypwr.pwr_2p2n_test import pwr_2p2n_test
from pypwr.pwr_anova_test import pwr_anova_test
from pypwr.pwr_chisq_test import pwr_chisq_test
from pypwr.pwr_norm_test import pwr_norm_test
from pypwr.pwr_p_test import pwr_p_test
from pypwr.pwr_r_test import pwr_r_test
from pypwr.pwr_f2_test import pwr_f2_test
from pypwr.plot_power_htest import plot_power_htest

name = "pypwr"



# -*- coding: utf-8 -*-
"""
Created in Jul 2022

@author: Ernest Namdar
"""
import sys
import numpy as np
from pypwr.cohen_ES import cohen_ES
from scipy.stats import t
from scipy.stats import nct
from scipy.stats import norm
from scipy.optimize import brentq
from pypwr.pwr_t_test import pwr_t_test
from pypwr.pwr_t2n_test import pwr_t2n_test
from pypwr.pwr_2p_test import pwr_2p_test
from pypwr.pwr_2p2n_test import pwr_2p2n_test
from pypwr.pwr_anova_test import pwr_anova_test
from pypwr.pwr_chisq_test import pwr_chisq_test
from pypwr.pwr_norm_test import pwr_norm_test
from pypwr.pwr_p_test import pwr_p_test
from pypwr.pwr_r_test import pwr_r_test
import matplotlib.pyplot as plt

def plot_power_htest(x):
    # initial checks
    if type(x) != dict:
        raise Exception("argument must be of type dictionary")
        sys.exit(0)
    pwr_methods = ["One-sample t test power calculation",
                   "Two-sample t test power calculation",
                   "Paired t test power calculation",
                   "t test power calculation",
                   "Difference of proportion power calculation for binomial distribution (arcsine transformation)",
                   "difference of proportion power calculation for binomial distribution (arcsine transformation)",
                   "Balanced one-way analysis of variance power calculation",
                   "Chi squared power calculation",
                   "Mean power calculation for normal distribution with known variance",
                   "proportion power calculation for binomial distribution (arcsine transformation)",
                   "approximate correlation power calculation (arctangh transformation)"]
    if not x["method"] in pwr_methods:
        raise Exception("the method ", x["method"], " is not supported. Supported methods include:", pwr_methods)
        sys.exit(0)
    # settings
    breaks = 20


    # case: One-sample, Two-sample or Paired t test
    if x["method"] == "One-sample t test power calculation" or x["method"] == "Two-sample t test power calculation" or x["method"] == "Paired t test power calculation":
        if x["method"] == "One-sample t test power calculation":
            x["test_type"]="one.sample"
        if x["method"] == "Two-sample t test power calculation":
            x["test_type"]="two.sample"
        if x["method"] == "Paired t test power calculation":
            x["test_type"]="paired"
        n = x["n"]
        n_upper = max(n*1.5, n+30) # upper at least 30 above n

        # generate data
        sample_sizes = list(np.arange(10, n_upper, (n_upper-10)/breaks))
        data_sample_size = np.array(sample_sizes)
        data_power = []
        for sample_size in sample_sizes:
            data_power.append(pwr_t_test(x["alternative"], x["test_type"], n=sample_size, d=x["d"], sig_level=x["sig_level"], power=None)["power"])
        data_power = np.array(data_power)
        #print(data_power)

        # create labels
        title_string = x["method"]
        #legend_string = '\n'.join(("tails =" + str(x["alternative"]), r"effect size d =", str(round(x["d"],4)), r"alpha =", str(x["sig_level"])))
        legend_string = "tails =" + str(x["alternative"]) + "\neffect size d =" + str(round(x["d"],4)) + "\nalpha =" + str(x["sig_level"])
        xlab_string = "sample size"
        ylab_string = "test power = 1 - "+ r'$\beta$'
        optimal_string = "optimal sample size \nn = " + str(np.ceil(n)) + "\n" + str(x["note"])


    # case: Two-sample t test with n1 and n2
    elif x["method"] == "t test power calculation":
        n = x["n1"] + x["n2"]
        n_upper = max(n*1.5, n+30) # upper at least 30 above n
        n_rel = x["n1"] / n # relative sample size; will be kept constant in claculations

        # generate data
        sample_sizes = list(np.arange(10, n_upper, (n_upper-10)/breaks))
        data_sample_size = np.array(sample_sizes)
        data_power = []
        for sample_size in sample_sizes:
            n1 = np.ceil(sample_size*n_rel)
            n2 = sample_size - n1
            if n1<2 or n2<2:
                data_power.append(None)
            else:
                data_power.append(pwr_t2n_test(alternative=x["alternative"], n1=n1, n2=n2, d=x["d"], sig_level=x["sig_level"], power=None)["power"])

        # create labels
        title_string = x["method"]
        #legend_string = '\n'.join(("tails =" + str(x["alternative"]), r"effect size d =", str(round(x["d"],4)), r"alpha =", str(x["sig_level"])))
        legend_string = "tails =" + str(x["alternative"]) + "\neffect size d =" + str(round(x["d"],4)) + "\nalpha =" + str(x["sig_level"]) + "\nn1/n2 = " + str(round(n_rel, 2))
        xlab_string = "sample size"
        ylab_string = "test power = 1 - "+ r'$\beta$'
        optimal_string = "optimal sample size \nn = " + str(x["n1"]) + '+' + str(x["n2"]) + '=' + str(n)


    # case: Difference of proportion (same sample size)
    elif x["method"] == "Difference of proportion power calculation for binomial distribution (arcsine transformation)":
        n = x["n"]
        n_upper = max(n*1.5, n+30) # upper at least 30 above n

        # generate data
        sample_sizes = list(np.arange(10, n_upper, (n_upper-10)/breaks))
        data_sample_size = np.array(sample_sizes)
        data_power = []
        for sample_size in sample_sizes:
            data_power.append(pwr_2p_test(x["alternative"], n=sample_size, h=x["h"], sig_level=x["sig_level"], power=None)["power"])
        data_power = np.array(data_power)
        #print(data_power)

        # create labels
        title_string = "Difference of proportion power calculation\nfor binomial distribution (arcsine transformation)"
        #legend_string = '\n'.join(("tails =" + str(x["alternative"]), r"effect size d =", str(round(x["d"],4)), r"alpha =", str(x["sig_level"])))
        legend_string = "tails =" + str(x["alternative"]) + "\neffect size h =" + str(round(x["h"],4)) + "\nalpha =" + str(x["sig_level"])
        xlab_string = "sample size"
        ylab_string = "test power = 1 - "+ r'$\beta$'
        optimal_string = "optimal sample size \nn = " + str(np.ceil(n)) + "\n" + str(x["note"])


    # case: difference of proportion (different sample size)
    elif x["method"] == "difference of proportion power calculation for binomial distribution (arcsine transformation)":
        n = x["n1"] + x["n2"]
        n_upper = max(n*1.5, n+30) # upper at least 30 above n
        n_rel = x["n1"] / n # relative sample size; will be kept constant in claculations

        # generate data
        sample_sizes = list(np.arange(10, n_upper, (n_upper-10)/breaks))
        data_sample_size = np.array(sample_sizes)
        data_power = []
        for sample_size in sample_sizes:
            n1 = np.ceil(sample_size*n_rel)
            n2 = sample_size - n1
            if n1<2 or n2<2:
                data_power.append(None)
            else:
                data_power.append(pwr_2p2n_test(alternative=x["alternative"], n1=n1, n2=n2, h=x["h"], sig_level=x["sig_level"], power=None)["power"])

        # create labels
        title_string = "Difference of proportion power calculation\nfor binomial distribution (arcsine transformation)"
        #legend_string = '\n'.join(("tails =" + str(x["alternative"]), r"effect size d =", str(round(x["d"],4)), r"alpha =", str(x["sig_level"])))
        legend_string = "tails =" + str(x["alternative"]) + "\neffect size h =" + str(round(x["h"],4)) + "\nalpha =" + str(x["sig_level"]) + "\nn1/n2 = " + str(round(n_rel, 2))
        xlab_string = "sample size"
        ylab_string = "test power = 1 - "+ r'$\beta$'
        optimal_string = "optimal sample size \nn = " + str(x["n1"]) + '+' + str(x["n2"]) + '=' + str(n)


    # case: ANOVA
    elif x["method"] == "Balanced one-way analysis of variance power calculation":
        n = x["n"]
        n_upper = max(n*1.5, n+30) # upper at least 30 above n

        # generate data
        sample_sizes = list(np.arange(10, n_upper, (n_upper-10)/breaks))
        data_sample_size = np.array(sample_sizes)
        data_power = []
        for sample_size in sample_sizes:
            data_power.append(pwr_anova_test(n=sample_size, k=x["k"], f=x["f"], sig_level=x["sig_level"], power=None)["power"])
        data_power = np.array(data_power)
        #print(data_power)

        # create labels
        title_string = "Balanced one-way analysis of variance \npower calculation"
        #legend_string = '\n'.join(("tails =" + str(x["alternative"]), r"effect size d =", str(round(x["d"],4)), r"alpha =", str(x["sig_level"])))
        legend_string = "groups k =" + str(x["k"]) + "\neffect size f =" + str(round(x["f"],4)) + "\nalpha =" + str(x["sig_level"])
        xlab_string = "sample size"
        ylab_string = "test power = 1 - "+ r'$\beta$'
        optimal_string = "optimal sample size \nn = " + str(np.ceil(n)) + "\n" + str(x["note"])




    # case: Chi Squared
    elif x["method"] == "Chi squared power calculation":
        n = x["N"]
        n_upper = max(n*1.5, n+30) # upper at least 30 above n

        # generate data
        sample_sizes = list(np.arange(10, n_upper, (n_upper-10)/breaks))
        data_sample_size = np.array(sample_sizes)
        data_power = []
        for sample_size in sample_sizes:
            data_power.append(pwr_chisq_test(N=sample_size, w=x["w"], df=x["df"], sig_level=x["sig_level"], power=None)["power"])
        data_power = np.array(data_power)
        #print(data_power)

        # create labels
        title_string = x["method"]
        #legend_string = '\n'.join(("tails =" + str(x["alternative"]), r"effect size d =", str(round(x["d"],4)), r"alpha =", str(x["sig_level"])))
        legend_string = "effect size w =" + str(round(x["w"],4)) + "\ndf =" + str(x["df"]) + "\nalpha =" + str(x["sig_level"])
        xlab_string = "sample size"
        ylab_string = "test power = 1 - "+ r'$\beta$'
        optimal_string = "optimal sample size \nN = " + str(np.ceil(n)) + "\n" + str(x["note"])



    # case: Normal distribution
    elif x["method"] == "Mean power calculation for normal distribution with known variance":
        n = x["n"]
        n_upper = max(n*1.5, n+30) # upper at least 30 above n

        # generate data
        sample_sizes = list(np.arange(10, n_upper, (n_upper-10)/breaks))
        data_sample_size = np.array(sample_sizes)
        data_power = []
        for sample_size in sample_sizes:
            data_power.append(pwr_norm_test(alternative=x["alternative"], n=sample_size, d=x["d"], sig_level=x["sig_level"], power=None)["power"])
        data_power = np.array(data_power)
        #print(data_power)

        # create labels
        title_string = "Mean power calculation for normal distribution\nwith known variance"
        #legend_string = '\n'.join(("tails =" + str(x["alternative"]), r"effect size d =", str(round(x["d"],4)), r"alpha =", str(x["sig_level"])))
        legend_string = "tails =" + str(x["alternative"]) + "\neffect size d =" + str(round(x["d"],4)) + "\nalpha =" + str(x["sig_level"])
        xlab_string = "sample size"
        ylab_string = "test power = 1 - "+ r'$\beta$'
        optimal_string = "optimal sample size \nn = " + str(np.ceil(n))


    # case: proportion
    elif x["method"] == "proportion power calculation for binomial distribution (arcsine transformation)":
        n = x["n"]
        n_upper = max(n*1.5, n+30) # upper at least 30 above n

        # generate data
        sample_sizes = list(np.arange(10, n_upper, (n_upper-10)/breaks))
        data_sample_size = np.array(sample_sizes)
        data_power = []
        for sample_size in sample_sizes:
            data_power.append(pwr_p_test(alternative=x["alternative"], n=sample_size, h=x["h"], sig_level=x["sig_level"], power=None)["power"])
        data_power = np.array(data_power)
        #print(data_power)

        # create labels
        title_string = "proportion power calculation\nfor binomial distribution (arcsine transformation)"
        #legend_string = '\n'.join(("tails =" + str(x["alternative"]), r"effect size d =", str(round(x["d"],4)), r"alpha =", str(x["sig_level"])))
        legend_string = "tails =" + str(x["alternative"]) + "\neffect size h =" + str(round(x["h"],4)) + "\nalpha =" + str(x["sig_level"])
        xlab_string = "sample size"
        ylab_string = "test power = 1 - "+ r'$\beta$'
        optimal_string = "optimal sample size \nn = " + str(np.ceil(n))


    # case: correlation
    elif x["method"] == "approximate correlation power calculation (arctangh transformation)":
        n = x["n"]
        n_upper = max(n*1.5, n+30) # upper at least 30 above n

        # generate data
        sample_sizes = list(np.arange(10, n_upper, (n_upper-10)/breaks))
        data_sample_size = np.array(sample_sizes)
        data_power = []
        for sample_size in sample_sizes:
            data_power.append(pwr_r_test(alternative=x["alternative"], n=sample_size, r=x["r"], sig_level=x["sig_level"], power=None)["power"])
        data_power = np.array(data_power)
        #print(data_power)

        # create labels
        title_string = "approximate correlation power calculation\n(arctangh transformation)"
        #legend_string = '\n'.join(("tails =" + str(x["alternative"]), r"effect size d =", str(round(x["d"],4)), r"alpha =", str(x["sig_level"])))
        legend_string = "tails =" + str(x["alternative"]) + "\nr =" + str(round(x["r"],4)) + "\nalpha =" + str(x["sig_level"])
        xlab_string = "sample size"
        ylab_string = "test power = 1 - "+ r'$\beta$'
        optimal_string = "optimal sample size \nn = " + str(np.ceil(n))
    else:
        raise Exception("unexpected case")
        sys.exit(0)


    # plot
    fig, ax = plt.subplots()
    plt.plot(data_sample_size,data_power, c="red", marker='o', markerfacecolor="black", markeredgecolor="black")
    plt.axvline(x=np.ceil(n), c="blue", linestyle=':')
    plt.xlabel(xlab_string)
    plt.ylabel(ylab_string)
    plt.title(title_string)
    plt.text(0.1, 0.9, legend_string, fontsize=12, transform=ax.transAxes)
    plt.text(0.85, 0.1, optimal_string, fontsize=12, transform=ax.transAxes)
    plt.gca().set_yticklabels([f'{x:.0%}' for x in plt.gca().get_yticks()])



# -*- coding: utf-8 -*-
"""
Created in Jul 2022

@author: Ernest Namdar
"""
import sys
import numpy as np
from pypwr.cohen_ES import cohen_ES
from scipy.stats import chi2
from scipy.stats import ncx2
from scipy.optimize import brentq

def pwr_chisq_test(w=None, N=None, df=None, sig_level=0.05, power=None):
    if sum(x is None for x in [w, N, df, power, sig_level]) != 1:
        raise Exception("exactly one of w, N, df, power, and sig_level must be NULL")
        sys.exit(0)
    if w is not None:
        if isinstance(w,str):
            w = cohen_ES(test="chisq",size=w)["effect_size"]
        if w<0:
            raise Exception("w must be positive")
            sys.exit(0)
    if (N is not None) and N<1:
        raise Exception("number of observations must be at least 1")
        sys.exit(0)
    if (sig_level is not None):
        #print(type(sig_level))
        if type(sig_level)!=int and type(sig_level)!=float:
            raise Exception("sig_level should be numerical")
            sys.exit(0)
        elif sig_level<0 or sig_level>1:
            raise Exception("sig_level should be in [0, 1]")
            sys.exit(0)
    if (power is not None):
        if type(power)!=int and type(power)!=float:
            raise Exception("power should be numerical")
            sys.exit(0)
        elif power<0 or power>1:
            raise Exception("power should be in [0, 1]")
            sys.exit(0)
    p_body = "ncx2.sf(chi2.isf(sig_level, df=df), df=df, nc=N*(w**2))"
    if power is None:
        power = eval(p_body)
    elif w is None:
        loc = {}
        target_function_def = "def target_function(w):return "+p_body+"-power"
        exec(target_function_def, loc)
        #print(loc)
        loc.update(globals())
        loc.update(locals())
        w = brentq(loc["target_function"], 1e-10, 1e+09)
    elif N is None:
        loc = {}
        target_function_def = "def target_function(N):return "+p_body+"-power"
        exec(target_function_def, loc)
        #print(loc)
        loc.update(globals())
        loc.update(locals())
        N = brentq(loc["target_function"], 1+1e-10, 1e+09)
    elif sig_level is None:
        loc = {}
        target_function_def = "def target_function(sig_level):return "+p_body+"-power"
        exec(target_function_def, loc)
        #print(loc)
        loc.update(globals())
        loc.update(locals())
        sig_level = brentq(loc["target_function"], 1e-10, 1-1e-10)
    else:
        raise Exception("internal error")
        sys.exit(0)
    NOTE = "N is the number of observations"
    METHOD = "Chi squared power calculation"
    # print("h=",h, "n=", n, "power=",power, "sig_level=", sig_level)
    return {"w":w, "N":N, "df":df, "sig_level":sig_level, "power":power,
            "method":METHOD, "note":NOTE}



import setuptools

with open("README.md", "r") as fh:

    long_description = fh.read()

setuptools.setup(

     name='pypwr',

     version='0.0.2',

     author="Ernest (Khashayar) Namdar",

     author_email="ernest.namdar@utoronto.ca",

     description="Python version of the pwr R package",

     long_description=long_description,

     long_description_content_type = "text/markdown",

     packages=setuptools.find_packages(),

     classifiers=[

         "Programming Language :: Python :: 3",

         "License :: OSI Approved :: MIT License",

         "Operating System :: OS Independent",

     ],

 )



# -*- coding: utf-8 -*-
"""
Created in Jul 2022

@author: Ernest Namdar
"""
import sys
import numpy as np
from pypwr.cohen_ES import cohen_ES
from scipy.stats import t
from scipy.stats import norm
from scipy.optimize import brentq

def pwr_r_test(alternative, n=None, r=None, sig_level=0.05, power=None):
    assert alternative in ["two.sided","less","greater"]
    alternative_options = ["less", "two.sided", "greater"]
    if sum(x is None for x in [n, r, power, sig_level]) != 1:
        raise Exception("exactly one of n, r, power, and sig.level must be NULL")
        sys.exit(0)
    if (r is not None) and isinstance(r,str):
        r = cohen_ES(test="r",size=r)["effect_size"]
    if (n is not None) and n<4:
        raise Exception("number of observations must be at least 4")
        sys.exit(0)
    if (sig_level is not None):
        #print(type(sig_level))
        if type(sig_level)!=int and type(sig_level)!=float:
            raise Exception("sig_level should be numerical")
            sys.exit(0)
        elif sig_level<0 or sig_level>1:
            raise Exception("sig_level should be in [0, 1]")
            sys.exit(0)
    if (power is not None):
        if type(power)!=int and type(power)!=float:
            raise Exception("power should be numerical")
            sys.exit(0)
        elif power<0 or power>1:
            raise Exception("power should be in [0, 1]")
            sys.exit(0)
    tside = alternative_options.index(alternative)
    if tside==1 and (r is not None):
        r = np.abs(r)
    if tside == 1:
        p_body = "norm.cdf(((np.arctanh(r) + r/(2*(n-1)))-(np.arctanh(np.sqrt((t.isf(sig_level/2, df=n-2))**2/((t.isf(sig_level/2, df=n-2))**2+n-2)))))*np.sqrt(n-3))+norm.cdf((-(np.arctanh(r) + r/(2*(n-1)))-(np.arctanh(np.sqrt((t.isf(sig_level/2, df=n-2))**2/((t.isf(sig_level/2, df=n-2))**2+n-2)))))*np.sqrt(n-3))"
        # ttt = t.isf(sig_level/2, df=n-2)
        # rc = np.sqrt(ttt**2/(ttt**2+n-2))
        # zr = np.arctanh(r) + r/(2*(n-1))
        # zrc = np.arctanh(rc) # + rc/(2*(n-1))
        # p_body = norm.cdf((zr-zrc)*np.sqrt(n-3))+norm.cdf((-zr-zrc)*np.sqrt(n-3))

    elif tside == 2:
        p_body = "norm.cdf(((np.arctanh(r) + r/(2*(n-1)))-(np.arctanh(np.sqrt((t.isf(sig_level, df=n-2))**2/((t.isf(sig_level, df=n-2))**2+n-2)))))*np.sqrt(n-3))"
        # ttt = t.isf(sig_level, df=n-2)
        # rc = np.sqrt(ttt**2/(ttt**2+n-2))
        # zr = np.arctanh(r) + r/(2*(n-1))
        # zrc = np.arctanh(rc) # + rc/(2*(n-1))
        # p_body = norm.cdf((zr-zrc)*np.sqrt(n-3))

    elif tside == 0:
        p_body = "norm.cdf(((np.arctanh(-r) - r/(2*(n-1)))-(np.arctanh(np.sqrt((t.isf(sig_level, df=n-2))**2/((t.isf(sig_level, df=n-2))**2+n-2)))))*np.sqrt(n-3))"

        # r = -r
        # ttt = t.isf(sig_level, df=n-2)
        # rc = np.sqrt(ttt**2/(ttt**2+n-2))
        # zr = np.arctanh(r) + r/(2*(n-1))
        # zrc = np.arctanh(rc) # + rc/(2*(n-1))
        # p_body = norm.cdf((zr-zrc)*np.sqrt(n-3))

    if power is None:
        power = eval(p_body)
    elif r is None:
        loc = {}
        target_function_def = "def target_function(r):return "+p_body+"-power"
        exec(target_function_def, loc)
        #print(loc)
        loc.update(globals())
        loc.update(locals())
        if tside==1:
            r = brentq(loc["target_function"], 1e-10, 1-1e-10)
        elif tside==0 or tside==2:
            r = brentq(loc["target_function"], -1+1e-10, 1-1e-10)
    elif n is None:
        loc = {}
        target_function_def = "def target_function(n):return "+p_body+"-power"
        exec(target_function_def, loc)
        #print(loc)
        loc.update(globals())
        loc.update(locals())
        n = brentq(loc["target_function"], 4+1e-10, 1e+09)
    elif sig_level is None:
        loc = {}
        target_function_def = "def target_function(sig_level):return "+p_body+"-power"
        exec(target_function_def, loc)
        #print(loc)
        loc.update(globals())
        loc.update(locals())
        sig_level = brentq(loc["target_function"], 1e-10, 1-1e-10)
    else:
        raise Exception("internal error")
        sys.exit(0)
    METHOD = "approximate correlation power calculation (arctangh transformation)"
    # print("h=",h, "n=", n, "power=",power, "sig_level=", sig_level)
    return {"n":n, "r":r, "sig_level":sig_level, "power":power,
            "alternative":alternative, "method":METHOD}



# -*- coding: utf-8 -*-
"""
Created in Jul 2022

@author: Ernest Namdar
"""
import sys
import numpy as np
from pypwr.cohen_ES import cohen_ES
from scipy.stats import norm
from scipy.optimize import brentq


def pwr_2p_test(alternative, h=None, n=None, sig_level=0.05, power=None):
    assert alternative in ["two.sided","less","greater"]
    alternative_options = ["less", "two.sided", "greater"]
    if sum(x is None for x in [h,n,power, sig_level]) != 1:
        raise Exception("exactly one of h, n, power, and sig_level must be NULL")
        sys.exit(0)
    if (h is not None) and isinstance(h,str):
        h = cohen_ES(test="p",size=h)["effect_size"]
    if (n is not None) and n<1:
        raise Exception("number of observations in each group must be at least 1")
        sys.exit(0)
    if (sig_level is not None):
        #print(type(sig_level))
        if type(sig_level)!=int and type(sig_level)!=float:
            raise Exception("sig_level should be numerical")
            sys.exit(0)
        elif sig_level<0 or sig_level>1:
            raise Exception("sig_level should be in [0, 1]")
            sys.exit(0)
    if (power is not None):
        if type(power)!=int and type(power)!=float:
            raise Exception("power should be numerical")
            sys.exit(0)
        elif power<0 or power>1:
            raise Exception("power should be in [0, 1]")
            sys.exit(0)
    tside = alternative_options.index(alternative)
    if tside==1 and (h is not None):
        h = np.abs(h)
    if tside == 1:
        p_body = "norm.sf(norm.isf(sig_level/2)-h*np.sqrt(n/2)) + norm.cdf(norm.ppf(sig_level/2)-h*np.sqrt(n/2))"
    elif tside == 2:
        p_body = "norm.sf(norm.isf(sig_level)-h*np.sqrt(n/2))"
    elif tside == 0:
        p_body = "norm.cdf(norm.ppf(sig_level)-h*np.sqrt(n/2))"
    if power is None:
        power = eval(p_body)
    elif h is None:
        loc = {}
        target_function_def = "def target_function(h):return "+p_body+"-power"
        exec(target_function_def, loc)
        #print(loc)
        loc.update(globals())
        loc.update(locals())
        if tside==1:
            h = brentq(loc["target_function"], 1e-10, 10)
        elif tside==0:
            h = brentq(loc["target_function"], -10,5)
        elif tside==2:
            h = brentq(loc["target_function"], -5, 10)
    elif n is None:
        loc = {}
        target_function_def = "def target_function(n):return "+p_body+"-power"
        exec(target_function_def, loc)
        #print(loc)
        loc.update(globals())
        loc.update(locals())
        n = brentq(loc["target_function"], 2+1e-10, 1e+09)
    elif sig_level is None:
        loc = {}
        target_function_def = "def target_function(sig_level):return "+p_body+"-power"
        exec(target_function_def, loc)
        #print(loc)
        loc.update(globals())
        loc.update(locals())
        sig_level = brentq(loc["target_function"], 1e-10, 1-1e-10)
    else:
        raise Exception("internal error")
        sys.exit(0)
    NOTE = "same sample sizes"
    METHOD = "Difference of proportion power calculation for binomial distribution (arcsine transformation)"
    # print("h=",h, "n=", n, "power=",power, "sig_level=", sig_level)
    return {"h":h, "n":n, "sig_level":sig_level, "power":power,
            "alternative":alternative, "method":METHOD, "note":NOTE}
